"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Tracker of jest status.
 */
class JestTracker {
    /**
     * Initialize trakcer.
     */
    constructor() {
        this._currentSpec = null;
        this._createEventHooks();
    }
    /**
     * Get current suite.
     * @param {jasmine.Suite} suite Suite (Must not set, for recursive process.)
     * @return {jasmine.Suite} Current suite.
     */
    getCurrentSuite(suite) {
        if (this._currentSpec) {
            return this._findOwnerSuite(this._currentSpec);
        }
        if (!suite) {
            return this.getCurrentSuite(jasmine.getEnv().topSuite());
        }
        const suites = suite.children.filter((child) => child instanceof jasmine.Suite);
        if (suites.length > 0) {
            return this.getCurrentSuite(suites.slice(-1)[0]);
        }
        return suite;
    }
    /**
     * Find a owner suite of a spec.
     * @param {jasmine.Spec}  spec  Spec.
     * @param {jasmine.Suite} suite Suite (Must not set, for recursive process.)
     * @return {jasmine.Suite} Owner suite of a spec
     */
    _findOwnerSuite(spec, suite) {
        if (!suite) {
            return this._findOwnerSuite(spec, jasmine.getEnv().topSuite());
        }
        if (suite.children.indexOf(spec) != -1) {
            return suite;
        }
        return suite.children
            .filter((child) => child instanceof jasmine.Suite)
            .map((child) => this._findOwnerSuite(spec, child))
            .find((child) => child);
    }
    /**
     * Process start event.
     * @param {jasmine.Spec} spec Starting spec.
     */
    _onStart(spec) {
        this._currentSpec = spec;
    }
    /**
     * Process finish event.
     */
    _onEnd() {
        this._currentSpec = null;
    }
    /**
     * Create event hooks.
     */
    _createEventHooks() {
        global.afterEach(() => this._onEnd());
        global.beforeAll(() => this._injectOnStartHookToAllSpec(jasmine.getEnv().topSuite()));
    }
    /**
     * Inject onStart hook to spec.
     * @param {jasmine.Suite | jasmine.Spec} context コンテキスト
     */
    _injectOnStartHookToAllSpec(context) {
        if (context instanceof jasmine.Spec) {
            const original = context.onStart;
            context.onStart = ((...args) => {
                this._onStart(context);
                return original(...args);
            });
        }
        else {
            context.children.forEach((child) => this._injectOnStartHookToAllSpec(child));
        }
    }
}
exports.default = JestTracker;
//# sourceMappingURL=jest_tracker.js.map